/*
 * MainBoard_UART.c
*************************************************************************
	Author: Alexander von Lhmann
	
	Hardware
   	processor:		AtMega644A
	clock:			32.768khz external RC oscillator

	Function		Pin		Word	Connected With

	Comm. Module 1
	TX1_M1			PA6				PD3 (RX1_M1)
	TX2_M1			PA7				PD2 (RX2_M1)
	RX_M1			PA5				PD1 (TX_M1)
	
	Comm. Module 2
	TX1_M2			PA3				PD3 (RX1_M2)
	TX2_M2			PA4				PD2 (RX2_M2)
	RX_M2			PA2				PD1 (TX_M2)
	
	Comm. Module 3
	TX1_M3			PA0				PD3 (RX1_M3)
	TX2_M3			PA1				PD2 (RX2_M3)
	RX_M3			PB0				PD1 (TX_M3)
	
	Comm. Module 4
	TX1_M4			PB2				PD3 (RX1_M4)
	TX2_M4			PB1				PD2 (RX2_M4)
	RX_M4			PB3				PD1 (TX_M4)

	UART:
	RXD0			PD0				TX_BT				
	TXD0			PD1				RX_BT
	RTS				PD2				RTS#BT
	CTS				PD3				CTS#BT
	
	SPI:
	MOSI			PB5				SDI ADC
	MISO			PB6				SDO ADC
	SS				PB4				+5V				
	SCK				PB7				SCK ADC
	Slave Select	PC0				CS ADC
	
	LED1			PC4				not running
	LED2			PC6			
	

*************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include "stdbool.h"
#include <string.h>

#define F_CPU 20E6 //Systemtakt in Hz
#define buffer_size 256
#include <util/delay.h>

#include "circbuf.h"
#include "SPI_functions.h"
#include "UART_functions.h"
#include "LTC2486_functions.h"
#include "Mainboard_UART_functions.h"




// Global VARIABLES

unsigned char byte_buffer;
uint16_t timer_ms_counter;								// x*10ms-Time-Value	

// Buffer Initialization
char volatile rxbufdata[buffer_size];
char volatile txbufdata[buffer_size];
CircBuf uart_rx_buffer = {
	rxbufdata, buffer_size, 0, 0, 0
};
CircBuf uart_tx_buffer = {
	txbufdata, buffer_size, 0, 0, 0
};


void LED2_on() {
	PORTC |= (1<<PINC6);
}

void LED2_off() {
	PORTC &= ~(1<<PINC6);
}

// Timer Compare Match A Interrupt (each ~10ms)
ISR(TIMER1_COMPA_vect) {
	timer_ms_counter++;
}

// UART RX-Complete interrupt: 
// Receiving data + buffering (saving)
ISR(USART0_RX_vect) {
	unsigned char data;
	data = UDR0;
	circbuf_add(&uart_rx_buffer, data);
}

// UART Data register empty interrupt:
// loading new data into UART sending register


ISR(USART0_UDRE_vect) {
	if (!circbuf_isempty(&uart_tx_buffer)) {				// Is buffer filled?
		UDR0 = circbuf_get(&uart_tx_buffer);
	}
	else {
		UCSR0B &= ~(1<<UDRIE0);							// Disable UDRIE- UART Data Register empty Interrupt 
	}
}


int main(void) {
	
	/* Variables */
	char controlbyte;
	bool speedmode = false;
	bool tempmeas = false;
	uint8_t active_channel= 0;
	uint8_t active_module= 1;
	volatile uint16_t adc_config =0;							// ADC-Config Data (for LTC2468)
	volatile uint16_t adc_value =0;							// Extracted ADC-Value
	volatile uint32_t adc_receive =0;						// Received ADC Value
	volatile uint16_t temp_value =0;						//Received and extracted Temp-Value
			
	/*********************** Initialization ***********************/
	//Port Directions
	
	DDRA= 0b11011011;				// NIRS-Module 1-3 Communication									
	DDRB= 0b10100110;				// NIRS-Module 3-4 Communication, SPI										
	DDRC= 0b01010001;				// SPI Slave select, LEDs										
	DDRD= 0b00001010;				// UART
	
	//Initializations
	UART_Init();					// UART initialization
	SPI_init();						// SPI initialization
	Timer_init();					// Timer initialization
	adc_config = LTC2486_create_config(active_module,speedmode,tempmeas,1);
	
	sei();							// Global Interrupt enable
	
    while(1) {	
			
		if (!circbuf_isempty(&uart_rx_buffer)) {						// Received Commands?	
			controlbyte = uart_ReceiveByte(&uart_rx_buffer);
			
			switch (controlbyte) {
				case 0xAA: {			
					// Start ADC Transmission
					adc_config = LTC2486_create_config(active_module,speedmode,tempmeas,1);
					adc_receive = LTC2486_SPI_get_value(adc_config);	
					adc_value = LTC2486_extract_temp(adc_receive);
					send_ADC_data(&uart_tx_buffer, active_module, active_channel, speedmode, adc_value, timer_ms_counter);		
					break;
				}
				case 0xBB: {
					// Stop ADC Transmission
					break;
				}
				case 0xCC: {
					// Config Mode: 
					break;					
				}
				case 0xDD: {
					// Transmit Temperature
					adc_config = LTC2486_create_config(0,false,true,1);
					adc_receive = LTC2486_SPI_get_value(adc_config);
					temp_value =LTC2486_extract_temp(adc_receive);
					send_ADC_temp(&uart_tx_buffer, temp_value);
					break;
				}
				case 0xEE: {
					adc_config = LTC2486_create_config(0,0,0,1);
					adc_receive =LTC2486_SPI_get_value(adc_config);
					send_ADC_raw(&uart_tx_buffer, adc_receive);
					break;
				}
			}
		}
		
    }
}